/*****************************************************************************
 *
 * utouch-grail - Multitouch Gesture Recognition Library
 *
 * Copyright (C) 2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#ifndef UTOUCH_GRAIL_RECOGNIZER_H_
#define UTOUCH_GRAIL_RECOGNIZER_H_

#include <list>
#include <map>
#include <memory>
#include <set>

#include <utouch/frame.h>

#include "utouch/grail.h"
#include "v3/forward.h"
#include "v3/subscription.h"

namespace utouch {
namespace grail {

class Recognizer {
 public:
  Recognizer(UGHandle* handle, const UFDevice device, UFWindowId window);
  ~Recognizer();

  UGStatus ActivateSubscription(UGSubscription* subscription);
  void DeactivateSubscription(UGSubscription* subscription);
  void ProcessFrameEvent(const UFEvent event);
  void UpdateTime(uint64_t time);
  uint64_t NextTimeout();
  UGStatus AcceptGesture(unsigned int id);
  UGStatus RejectGesture(unsigned int id);

  UGHandle* handle() const { return handle_; }
  float device_x_res() const { return device_x_res_; }
  float device_y_res() const { return device_y_res_; }
  bool device_direct() const { return device_direct_; }

  Recognizer(const Recognizer&) = delete;
  Recognizer& operator=(const Recognizer&) = delete;

 private:
  UGHandle* const handle_;
  const UFDevice device_;
  const UFWindowId window_id_;
  float device_x_res_; /* Units of pixel/mm */
  float device_y_res_;
  bool device_direct_;
  bool atomic_;
  std::set<UGSubscription*> subscriptions_[5];
  std::set<SharedGesture> unaccepted_gestures_;
  std::set<SharedGesture> accepted_gestures_;
  std::map<UFTouchId, uint64_t> start_times_;
  TouchSet unaccepted_touches_;
  TouchSet accepted_touches_;
  TouchSet free_touches_;
  unsigned int num_subscriptions_;

  void HandleNewTouchForAcceptedGesture(UFTouchId touch_id,
                                        const SharedGesture& gesture);
  void HandleNewTouchForUnacceptedGesture(UFTouchId touch_id,
                                          const SharedGesture& gesture);
  void MatchSubscriptionsForEvent(const UFEvent event);
  void MatchSubscriptionsForTouch(UFTouchId touch_id);
  void MatchOneTouchGestures(UFTouchId touch_id);
  void MatchTwoTouchGestures(UFTouchId touch_id);
  void MatchThreeTouchGestures(UFTouchId touch_id);
  void MatchFourTouchGestures(UFTouchId touch_id);
  void MatchFiveTouchGestures(UFTouchId touch_id);
  void ProcessEvent(const UFEvent& event);
  void CheckConstructionFinished(uint64_t time);
  void RejectGesture(SharedGesture gesture);
};

} // namespace grail
} // namespace utouch
#endif // UTOUCH_GRAIL_RECOGNIZER_H_
