/*****************************************************************************
 *
 * utouch-frame - Touch Frame Library
 *
 * Copyright (C) 2010-2011 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#include "common/servertime.h"

#include <stdio.h>
#include <string.h>

XSyncCounter get_server_time_counter(Display *display) {
  int event_base;
  int error_base;
  int major_version;
  int minor_version;
  XSyncSystemCounter *counters;
  int num_system_counters;
  int i;

  if (XSyncQueryExtension(display, &event_base, &error_base) != True) {
    fprintf(stderr, "Warning: failed to query sync extension\n");
    return -1;
  }

  if (XSyncInitialize(display, &major_version, &minor_version) != True) {
    fprintf(stderr, "Warning: failed to initialize sync extension\n");
    return -1;
  }

  counters = XSyncListSystemCounters(display, &num_system_counters);
  for (i = 0; i < num_system_counters; ++i) {
    if (strcmp(counters[i].name, "SERVERTIME") == 0) {
      XSyncCounter server_time_counter = counters[i].counter;
      XSyncFreeSystemCounterList(counters);
      return server_time_counter;
    }
  }

  fprintf(stderr, "Warning: failed to find SERVERTIME counter\n");
  XSyncFreeSystemCounterList(counters);
  return -1;
}

static uint64_t get_server_time(Display *display, XSyncCounter counter) {
  XSyncValue time;

  if (XSyncQueryCounter(display, counter, &time) != True) {
    fprintf(stderr, "Warning: failed to get server time\n");
    return 0;
  }

  return (uint64_t)XSyncValueHigh32(time) << 32 | XSyncValueLow32(time);
}

void update_time(UGHandle handle, Display *display, XSyncCounter counter) {
  uint64_t server_time = get_server_time(display, counter);

  grail_update_time(handle, server_time);
}

void set_timeout(UGHandle handle, Display *display, XSyncCounter counter) {
  uint64_t timeout = grail_next_timeout(handle);
  if (timeout) {
    XSyncWaitCondition condition;

    condition.trigger.counter = counter;
    condition.trigger.value_type = XSyncAbsolute;
    XSyncIntsToValue(&condition.trigger.wait_value, timeout & 0xffffffff,
                     timeout & 0xffffffff00000000);
    condition.trigger.test_type = XSyncPositiveComparison;
    XSyncIntToValue(&condition.event_threshold, 0);
    if (!XSyncAwait(display, &condition, 1))
      fprintf(stderr, "Warning: failed to set wait trigger\n");
    XFlush(display);
  }
}
